@@ -28,14 +28,17 @@ with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "pulse-manager.h"
 #include "dbus-menu-manager.h"
 
+#define RECONNECT_DELAY 5
+
 static GHashTable *sink_hash = NULL;
 static SoundServiceDbus *dbus_service = NULL;
 static gint DEFAULT_SINK_INDEX = -1;
 static gboolean pa_server_available = FALSE;
-// PA related
+static gint reconnect_idle_id = 0;
 static pa_context *pulse_context = NULL;
 static pa_glib_mainloop *pa_main_loop = NULL;
 static void context_state_callback(pa_context *c, void *userdata);
+static gboolean reconnect_to_pulse();
 static void pulse_sink_info_callback(pa_context *c, const pa_sink_info *sink_info, int eol, void *userdata);
 static void context_success_callback(pa_context *c, int success, void *userdata);
 static void pulse_sink_input_info_callback(pa_context *c, const pa_sink_input_info *info, int eol, void *userdata);
@@ -43,7 +46,6 @@ static void pulse_server_info_callback(pa_context *c, const pa_server_info *info
 static void update_sink_info(pa_context *c, const pa_sink_info *info, int eol, void *userdata);
 static void destroy_sink_info(void *value);
 static gboolean determine_sink_availability();
-static void reconnect_to_pulse();
 
 static gboolean has_volume_changed(const pa_sink_info* new_sink, sink_info* cached_sink);
 static pa_cvolume construct_mono_volume(const pa_cvolume* vol);
@@ -79,7 +81,42 @@ Needed for testing - bah!
 **/
 pa_context* get_context()
 {
-    return pulse_context;
+  return pulse_context;
+}
+
+static gboolean
+reconnect_to_pulse()
+{
+  g_debug("Attempt to reconnect to pulse");
+  // reset
+  if (pulse_context != NULL) {
+    pa_context_unref(pulse_context);
+    pulse_context = NULL;
+  }
+
+  if (sink_hash != NULL) {
+   g_hash_table_destroy(sink_hash);
+    sink_hash = NULL;
+  }
+  pulse_context = pa_context_new( pa_glib_mainloop_get_api( pa_main_loop ),
+                                  "ayatana.indicator.sound" );
+  g_assert(pulse_context);
+  sink_hash = g_hash_table_new_full( g_direct_hash, g_direct_equal,
+                                     NULL,
+                                     destroy_sink_info );
+  // Establish event callback registration
+  pa_context_set_state_callback (pulse_context, context_state_callback, NULL);
+  int result = pa_context_connect (pulse_context, NULL, PA_CONTEXT_NOFAIL, NULL);
+
+  if (result < 0) {
+    g_warning ("Failed to connect context: %s",
+               pa_strerror (pa_context_errno (pulse_context)));
+  }
+  // we always want to cancel any continious callbacks with the existing timeout
+  // if the connection failed the new context created above will catch any updates
+  // to do with the state of pulse and thus take care of business.
+  reconnect_idle_id = 0;  
+  return FALSE;
 }
 
 /**
@@ -99,35 +136,6 @@ void close_pulse_activites()
     g_debug("I just closed communication with Pulse");
 }
 
-/** 
-reconnect_to_pulse()
-In the event of Pulseaudio flapping in the wind handle gracefully without
-memory leaks !
-*/
-static void reconnect_to_pulse()
-{
-    // reset
-    if (pulse_context != NULL){
-        g_debug("freeing the pulse context");
- 	    pa_context_unref(pulse_context);
-        pulse_context = NULL;
-   	}
-    
-    if(sink_hash != NULL){
-        g_hash_table_destroy(sink_hash);
-        sink_hash = NULL;
-    }
-
-    // reconnect
-	pulse_context = pa_context_new(pa_glib_mainloop_get_api(pa_main_loop), "ayatana.indicator.sound");
-	g_assert(pulse_context);   
-    sink_hash = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, destroy_sink_info);
-    // Establish event callback registration
-	pa_context_set_state_callback(pulse_context, context_state_callback, NULL);
-    dbus_menu_manager_update_pa_state(FALSE, FALSE, FALSE, 0);
-	pa_context_connect(pulse_context, NULL, PA_CONTEXT_NOFAIL, NULL);        
-}
-
 /**
 destroy_sink_info()
 item destructor method for the sink_info hash
@@ -550,42 +558,50 @@ static void subscribed_events_callback(pa_context *c, enum pa_subscription_event
 
 static void context_state_callback(pa_context *c, void *userdata) {
 	switch (pa_context_get_state(c)) {
-        case PA_CONTEXT_UNCONNECTED:
-			g_debug("unconnected");
+      case PA_CONTEXT_UNCONNECTED:
+  			g_debug("unconnected");
 			break;
-        case PA_CONTEXT_CONNECTING:
-			g_debug("connecting - waiting for the server to become available");
+      case PA_CONTEXT_CONNECTING:
+  			g_debug("connecting - waiting for the server to become available");
 			break;
-        case PA_CONTEXT_AUTHORIZING:
-			g_debug("authorizing");
+      case PA_CONTEXT_AUTHORIZING:
+  			g_debug("authorizing");
 			break;
-        case PA_CONTEXT_SETTING_NAME:
-			g_debug("context setting name");
+      case PA_CONTEXT_SETTING_NAME:
+  			g_debug("context setting name");
 			break;
-        case PA_CONTEXT_FAILED:
-			g_warning("FAILED to retrieve context - Is PulseAudio Daemon running ?");
-            pa_server_available = FALSE;
-            reconnect_to_pulse();
+      case PA_CONTEXT_FAILED:
+        g_warning("PA_CONTEXT_FAILED - Is PulseAudio Daemon running ?");
+        pa_server_available = FALSE;
+        dbus_menu_manager_update_pa_state(TRUE,
+                                          pa_server_available,
+                                          default_sink_is_muted(),
+                                          get_default_sink_volume());
+          
+        if (reconnect_idle_id == 0){
+          reconnect_idle_id = g_timeout_add_seconds (RECONNECT_DELAY,
+                                                     reconnect_to_pulse,
+                                                     NULL);                                                      
+        }     
+      break;        
+      case PA_CONTEXT_TERMINATED:
 			break;
-        case PA_CONTEXT_TERMINATED:
-			g_debug("context terminated");
-			break;
-        case PA_CONTEXT_READY:
-			g_debug("PA daemon is ready");
-            pa_operation *o;
+      case PA_CONTEXT_READY:
+			  g_debug("PA daemon is ready");
+        pa_operation *o;
 
-            pa_context_set_subscribe_callback(c, subscribed_events_callback, userdata);
+        pa_context_set_subscribe_callback(c, subscribed_events_callback, userdata);
 
-            if (!(o = pa_context_subscribe(c, (pa_subscription_mask_t)
-                                           (PA_SUBSCRIPTION_MASK_SINK|
-                                            PA_SUBSCRIPTION_MASK_SINK_INPUT|
-                                            PA_SUBSCRIPTION_MASK_SERVER), NULL, NULL))) {
-                g_warning("pa_context_subscribe() failed");
-                return;
-            }
-            pa_operation_unref(o);
-            
-            gather_pulse_information(c, userdata);
+        if (!(o = pa_context_subscribe(c, (pa_subscription_mask_t)
+                                       (PA_SUBSCRIPTION_MASK_SINK|
+                                        PA_SUBSCRIPTION_MASK_SINK_INPUT|
+                                        PA_SUBSCRIPTION_MASK_SERVER), NULL, NULL))) {
+            g_warning("pa_context_subscribe() failed");
+            return;
+        }
+        pa_operation_unref(o);
+        
+        gather_pulse_information(c, userdata);
 
 			break;
     }
